<script setup lang="ts">
// 引入 Three.js 和相关模块
import {resPath} from '@/utils/config'; // 引入资源路径配置文件

import gynn from '@/assets/svg/gynn.vue'; // 引入自定义组件
import * as THREE from 'three/webgpu'; // 使用 WebGPU 渲染器
import { step, normalWorld, output, texture, vec2, vec3, vec4, normalize, positionWorld, bumpMap, cameraPosition, color, uniform, mix, uv, max } from 'three/tsl'; // 引入 TSL 工具

import { OrbitControls } from 'three/examples/jsm/controls/OrbitControls'; // 引入轨道控制器
import * as TWEEN from 'three/examples/jsm/libs/tween.module.js'; // 引入 Tween 动画库

import { ref, onMounted, onUnmounted, watch } from 'vue'; // Vue 生命周期钩子
import { time } from 'three/tsl'; // 引入时间变量

let camera: any; // 相机
let scene: any; // 场景
let renderer: any; // 渲染器
let controls: any; // 轨道控制器
let globe: any; // 地球模型
let clock: any; // 时钟，用于动画
const curtain: any = ref(null); // 加载幕布引用

const canvasWidth = ref(0); // 响应式变量，存储 canvas 的宽度
const canvasHeight = ref(0); // 响应式变量，存储 canvas 的高度

onMounted(() => {

    const canvas: any = document.getElementById('containerEarth'); // 获取容器元素
    clock = new THREE.Clock(); // 初始化时钟

    // 创建透视相机
    camera = new THREE.PerspectiveCamera(45, canvas?.clientWidth / canvas?.clientHeight, 0.1, 1000);
    camera.position.set(0, 0, 56); // 设置相机初始位置（X, Y, Z）

    // 创建场景
    scene = new THREE.Scene();

    // 添加太阳光源（平行光）
    const sun = new THREE.DirectionalLight('#ffffff', 2); // 白色平行光，强度为 2
    sun.position.set(0, 0, 3); // 设置光源位置
    scene.add(sun); // 将光源添加到场景中

    // 定义大气层颜色和粗糙度参数
    const atmosphereDayColor = uniform(color('#4db2ff')); // 白天大气颜色
    const atmosphereTwilightColor = uniform(color('#bc490b')); // 黄昏大气颜色
    const roughnessLow = uniform(0.25); // 最低粗糙度
    const roughnessHigh = uniform(0.35); // 最高粗糙度

    // 纹理加载管理器
    const loadingManager = new THREE.LoadingManager();
    const textureLoader = new THREE.TextureLoader(loadingManager);

    // 加载地球纹理
    const dayTexture = textureLoader.load(resPath+'img/Three/earth_day.jpg'); // 白天纹理
    dayTexture.colorSpace = THREE.SRGBColorSpace; // 设置颜色空间
    dayTexture.anisotropy = 8; // 提高纹理质量
    dayTexture.offset.y = .12; // 偏移纹理

    const nightTexture = textureLoader.load(resPath+'img/Three/earth_night.jpg'); // 夜晚纹理
    nightTexture.colorSpace = THREE.SRGBColorSpace;
    nightTexture.anisotropy = 8;
    nightTexture.offset.y = .12;

    const bumpRoughnessCloudsTexture = textureLoader.load(resPath+'img/Three/earth_bump_roughness_clouds.jpg'); // 凹凸、粗糙度
    bumpRoughnessCloudsTexture.anisotropy = 8;
    bumpRoughnessCloudsTexture.offset.y = .12;
    bumpRoughnessCloudsTexture.wrapS = THREE.RepeatWrapping; // 设置纹理水平重复
    bumpRoughnessCloudsTexture.wrapT = THREE.RepeatWrapping; // 设置纹理垂直重复

   
    // 计算法线方向和菲涅尔效应
    const viewDirection = positionWorld.sub(cameraPosition).normalize(); // 视线方向
    const fresnel = viewDirection.dot(normalWorld).abs().oneMinus().toVar(); // 菲涅尔效果

    // 计算太阳方向
    const sunOrientation = normalWorld.dot(normalize(sun.position)).toVar();

    // 大气层颜色混合
    const atmosphereColor = mix(
        atmosphereTwilightColor,
        atmosphereDayColor,
        sunOrientation.smoothstep(-0.25, 0.75)
    );

    // 创建地球材质
    const globeMaterial = new THREE.MeshStandardNodeMaterial();
    const timeFactor = time.mul(-0.005); // 反转时间方向（负值）并控制速度
    const cloudsStrength = texture( bumpRoughnessCloudsTexture, uv().add(vec2(timeFactor, 0)) ).b.smoothstep( 0.2, 1 );
    globeMaterial.colorNode = mix(texture(dayTexture), vec3(1), cloudsStrength.mul(2));

    // 粗糙度计算
    const roughness = max(
        texture(bumpRoughnessCloudsTexture).g,
        step(0.01, cloudsStrength)
    );
    globeMaterial.roughnessNode = roughness.remap(0, 1, roughnessLow, roughnessHigh);

    // 混合白天和夜晚纹理
    const night = texture(nightTexture);
    const dayStrength = sunOrientation.smoothstep(-0.25, 0.5);

    const atmosphereDayStrength = sunOrientation.smoothstep(-0.5, 1);
    const atmosphereMix = atmosphereDayStrength.mul(fresnel.pow(2)).clamp(0, 1);

    let finalOutput = mix(night.rgb, output.rgb, dayStrength);
    finalOutput = mix(finalOutput, atmosphereColor, atmosphereMix);

    globeMaterial.outputNode = vec4(finalOutput, output.a);

    // 凹凸贴图
    const bumpElevation = max(
        texture(bumpRoughnessCloudsTexture).r,
        cloudsStrength
    );
    globeMaterial.normalNode = bumpMap(bumpElevation);

    // 创建地球几何体
    const sphereGeometry = new THREE.SphereGeometry(2, 64, 64); // 半径为 2 的球体
    globe = new THREE.Mesh(sphereGeometry, globeMaterial); // 创建地球网格
    scene.add(globe); // 将地球添加到场景中

    // 创建大气层材质
    const atmosphereMaterial = new THREE.MeshBasicNodeMaterial({ side: THREE.BackSide, transparent: true });
    let alpha = fresnel.remap(0.73, 1, 1, 0).pow(3);
    alpha = alpha.mul(sunOrientation.smoothstep(-0.5, 1));
    atmosphereMaterial.outputNode = vec4(atmosphereColor, alpha);

    const atmosphere = new THREE.Mesh(sphereGeometry, atmosphereMaterial); // 创建大气层网格
    atmosphere.scale.setScalar(1.04); // 放大比例，使大气层包裹地球
    scene.add(atmosphere); // 将大气层添加到场景中

    // 创建天空盒
    const urls = [
        resPath+'img/Three/posX.jpg', // 正面
        resPath+'img/Three/negX.jpg', // 背面
        resPath+'img/Three/posY.jpg', // 顶面
        resPath+'img/Three/negY.jpg', // 底面
        resPath+'img/Three/posZ.jpg', // 右面
        resPath+'img/Three/negZ.jpg'  // 左面
    ];
    const loader = new THREE.CubeTextureLoader(loadingManager);
    const textureTK = loader.load(urls); // 加载天空盒纹理
    scene.background = textureTK; // 设置场景背景为天空盒

    // 创建渲染器
    renderer = new THREE.WebGPURenderer();

    renderer.setAnimationLoop(animate); // 开始动画循环
    if (canvas) {
        canvas.appendChild(renderer.domElement); // 将渲染器附加到 DOM 中
        canvasWidth.value = canvas.clientWidth; // 初始化宽度
    }
    updateRendererSize(); // 初始化渲染器尺寸



    // 创建轨道控制器
    controls = new OrbitControls(camera, renderer.domElement);
    controls.enableDamping = true; // 启用阻尼效果
    controls.minDistance = 5; // 最小缩放距离
    controls.maxDistance = 5000; // 最大缩放距离
    controls.enableZoom = false; // 禁用缩放
    controls.enablePan = false; // 禁用平移
    controls.minPolarAngle = Math.PI / 2; // 限制上下旋转角度
    controls.maxPolarAngle = Math.PI / 2;

    // 创建 Tween 动画
    const tween = new TWEEN.Tween({ distance: 40, rotation: 0 }) // 初始状态
        .to({ distance: 6.5, rotation: Math.PI * 2 }, 2000) // 结束状态，持续 2000 毫秒
        .easing(TWEEN.Easing.Quadratic.InOut) // 缓动函数
        .onUpdate((obj: any) => {
            // 更新相机位置
            const angle = obj.rotation;
            const x = Math.cos(angle) * obj.distance;
            const z = Math.sin(angle) * obj.distance;
            camera.position.set(x, 0, z);

            // 更新轨道控制器
            controls.update();
        });

    // 资源加载完成后的回调
    loadingManager.onLoad = function () {
        curtain.value.classList.add('bj1'); // 隐藏加载幕布
        tween.start(); // 开始动画
    };
    // 动画循环
    let lastRenderTime = 0; // 上次渲染的时间戳
    const renderInterval = 8; // 每隔 16 毫秒（约 60 FPS）渲染一次

    // 动画循环
    async function animate(currentTime: number) {
        if (currentTime - lastRenderTime >= renderInterval) {


            const delta = clock.getDelta(); // 获取时间间隔
            globe.rotation.y += delta * 0.1; // 旋转地球

            TWEEN.update(); // 更新 Tween 动画
            controls.update(); // 更新轨道控制器
            renderer.render(scene, camera); // 渲染场景
        }
    }

    // 更新渲染器尺寸
    function updateRendererSize() {
        const width = canvas?.clientWidth || 0;
        const height = canvas?.clientHeight || 0; // 根据宽高比计算高度
        renderer.setSize(width, height); // 更新渲染器尺寸
        camera.aspect = width / height; // 更新相机宽高比
        camera.updateProjectionMatrix(); // 更新投影矩阵
    }
    // 创建 ResizeObserver 来监听 canvas 尺寸变化
    const resizeObserver = new ResizeObserver(() => {
        if (canvas) {
            canvasWidth.value = canvas.clientWidth; // 更新宽度
            canvasHeight.value = canvas.clientHeight; // 更新高度
        }
    });

    if (canvas) {
        resizeObserver.observe(canvas); // 开始观察
    }
    // 监听 canvasWidth 的变化
    watch(canvasWidth, () => {
        updateRendererSize(); // 当宽度变化时更新渲染器尺寸
    });
    typeWriterEffect(); // 启动打字机效果
    // 组件卸载时清理资源
    onUnmounted(() => {
        if (canvas) {
            resizeObserver.unobserve(canvas); // 停止观察
        }
        if (renderer) {
            renderer.dispose(); // 释放渲染器资源
            renderer.domElement.remove(); // 移除渲染器 DOM 元素
        }
        scene = null; // 清空场景
        camera = null; // 清空相机
    });
    
});
// 打字机效果
const typeWriterEffect = () => {
    const text = "希望我们的存在，能够帮助到闪闪发光的你";
    const title1Element = document.querySelector('.title1') as HTMLElement;
    let index = 0;

    const type = () => {
        if (index < text.length) {
            title1Element.textContent += text.charAt(index);
            index++;
            setTimeout(type, 100); // 每隔 100ms 显示一个字符
        } else {
            title1Element.style.borderRight = 'none'; // 打字完成后移除光标
        }
    };

    type();
};

</script>

<template>
    <!-- 加载幕布 -->
    <div class="curtain">
        <div ref="curtain" class="bj"></div> <!-- 幕布背景 -->
        <div class="title">
            <gynn /> <!-- 自定义组件 -->
        </div>
        <div class="title1">
            <!-- 希望我们的存在，能够帮助到闪闪发光的你  -->
        </div>
    </div>
    <!-- 地球容器 -->
    <div id="containerEarth" class="containerEarth"></div>
</template>

<style lang="scss" scoped>
.containerEarth {
    width: 100%; // 宽度占满父容器
    // aspect-ratio: 16 / 9; // 宽高比
    height: 100vh; // 高度占满视口
    z-index: 700; // 设置层级

    // 当窗口宽度小于 1000px 时的样式
    @media (max-width: 1000px) {
        aspect-ratio: 16 / 9; // 宽高比

    }
}

.curtain {
    position: absolute; // 绝对定位
    width: 100%; // 宽度占满父容器
    // aspect-ratio: 16 / 9;
    height: 100vh; // 高度占满视口
    pointer-events: none; // 禁止鼠标交互
    z-index: 600; // 设置层级
    display: flex; // 使用 Flexbox 布局
    align-items: center; // 垂直居中
    justify-content: center; // 水平居中
    flex-direction: column; // 子元素垂直排列

    @media (max-width: 1000px) {
        aspect-ratio: 16 / 9; // 宽高比

    }

    .bj {
        position: absolute; // 绝对定位
        width: 100%; // 宽度占满父容器
        height: 100%; // 高度占满父容器
        background-color: rgb(0, 0, 0); // 黑色背景
    }

    .bj1 {
        transition: all 2s; // 过渡效果
        opacity: 0; // 隐藏幕布
    }

    .title {
        pointer-events: all; // 启用鼠标交互
        z-index: 999; // 设置层级
    }

    .title1 {
        z-index: 999; // 设置层级
        color: #fff; // 白色文字
        font-size: 14px; // 字体大小
        font-weight: 400; // 字体粗细
        margin-top: 20px; // 上边距


        text-align: center;
        white-space: nowrap;
        /* 禁止换行 */
        overflow: hidden;
        /* 隐藏未显示的文字 */
        border-right: 2px solid #fff;
        /* 模拟光标效果 */
        animation: blink-caret 0.75s step-end infinite;
        /* 光标闪烁动画 */
    }

    /* 光标闪烁动画 */
    @keyframes blink-caret {

        from, to {
            border-color: transparent;
        }

        50% {
            border-color: #fff;
        }
    }
}
</style>